/*
 * Copyright (c) 2004
 *	Hartmut Brandt
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/ats_cc/test_cc.h,v 1.6 2004/07/16 18:57:10 brandt Exp $
 */
#include <sys/types.h>

/*
 * Interface to CC implementation
 */
struct exstat {
	struct atm_exstatus		status;
	struct atm_exstatus_ep		*ep;
	struct atm_exstatus_port	*port;
	struct atm_exstatus_conn	*conn;
	struct atm_exstatus_party	*party;
};

struct uni_msg;
struct cc {
	void	(*init)(u_int);
	void	(*reset)(void);
	void	(*start)(void);
	void	(*work)(void);
	void	(*dump)(void);
	void	(*uni_to_cc)(u_int, u_int, u_int, struct uni_msg *);
	void	(*api_to_cc)(const char *, u_int, struct uni_msg *);

	void	*(*create_user)(const char *);
	void	(*destroy_user)(const char *);

	void	*(*create_port)(u_int);
	void	(*destroy_port)(u_int);
	void	(*clear_port)(u_int);
	void	(*port_set_info)(const struct atm_port_info *);
	void	(*port_get_info)(u_int, struct atm_port_info *);
	void	(*list_ports)(u_int *, u_int **);
	void	(*admin_port)(u_int, int);

	int	(*register_addr)(u_int, const struct uni_addr *);
	int	(*unregister_addr)(u_int, const struct uni_addr *);
	void	(*query_addr_list)(u_int, struct uni_addr **, u_int **,
		    u_int *);

	void	(*unmanage)(void);
	struct exstat	*(*get_exstat)(void);
	void	(*free_exstat)(struct exstat *);
	void	(*set_log)(u_int);
};
extern const struct cc user_cc;
extern const struct cc kernel_cc;

/* handle data output to API */
void handle_api_data(const char *epname, u_int op, void *, size_t);

/* handle response to API */
void handle_api_response(const char *epname, int e, u_int d, void *, size_t);

/* handle message from CC to lower layer */
void handle_uni_data(u_int portno, u_int sig, u_int cookie, struct uni_msg *);
