/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/libats/ats_report.c,v 1.6 2003/09/17 09:55:20 hbb Exp $
 */

#include <stdarg.h>
#include "ats.h"

/* current trace level */
u_int trace_level;

/* verbosity flags */
uint8_t	verb_flag[256 / 8];

/* reporting level */
u_int report_level = 3;

/* number of problems encountered in current test */
static u_int current_problems;

/* number of tests ok and failed */
static u_int tests_ok;
static u_int tests_failed;
static u_int tests_skipped;

/*
 * problem report
 */
void
report_problem(const char *fmt, ...)
{
	va_list ap;

	current_problems++;

	if (report_level < 3)
		return;

	printf("PROBLEM(%s): ", current_test_name);
	va_start(ap, fmt);
	vprintf(fmt, ap);
	va_end(ap);
	printf("\n");

	if (trace_level > 2) {
		fprintf(stderr, "  in file %s, %u\n",
		    input->pos.name, input->pos.lno);
		print_stack(input->stack);
	}

}

/*
 * Report at end of test
 */
void
report_test_result(void)
{

	if (skip_test) {
		tests_skipped++;
		if (report_level >= 2)
			printf("TEST SKIPPED: %s\n", current_test_name);
	} else if (current_problems != 0) {
		tests_failed++;
		if (report_level >= 2)
			printf("TEST FAILED: %s\n", current_test_name);
	} else {
		tests_ok++;
		if (report_level >= 2)
			printf("TEST OK: %s\n", current_test_name);
	}
}

/*
 * Init a test
 */
void
report_test_init(void)
{
	current_problems = 0;
}

/*
 * Report total results
 */
u_int
report_total(void)
{
	if (report_level >= 1)
		printf("%u test%s OK; %u test%s FAILED; %u test%s SKIPPED\n",
		    tests_ok, tests_ok == 1 ? "" : "s",
		    tests_failed, tests_failed == 1 ? "" : "s",
		    tests_skipped, tests_skipped == 1 ? "" : "s");

	return (tests_failed);
}

/*
 * Write a trace message
 */
void
trace(u_int level, const char *fmt, ...)
{
	va_list ap;

	if (trace_level >= level) {
		va_start(ap, fmt);
		printf("%*s", (int)level * 2, "");
		vprintf(fmt, ap);
		printf("\n");
	}
}
