#
# Copyright (c) 2001-2004
#	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
# 	All rights reserved.
#
# Author: Hartmut Brandt <harti@freebsd.org>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Begemot: atmsupport/libunitcl/genmsgc.awk,v 1.8 2004/07/08 13:33:41 brandt Exp $
#

function begin() {
	foo = 1
}

function first_entry() {
	print "/* This file was created automatically"
	print " * Source file: " id
	print " */"
	print ""
	print "#include <stdio.h>"
	print "#include <stdlib.h>"
	print "#include <errno.h>"
	print "#include <string.h>"
	print "#include <unistd.h>"
	print ""
	print "#include \"inctcl.h\""
	print ""
	print "#include <netnatm/unimsg.h>"
	print "#include <netnatm/msg/unistruct.h>"
	print "#include <netnatm/api/unisap.h>"
	print "#include <netnatm/msg/unimsglib.h>"
	print ""
	print "#include \"unitcl.h\""
	print "#include \"unitcl_priv.h\""
}

function end() {
	gen_regfunc()
}

function start_message() {
	foo = 1
}

function end_message() {
	gen_fmt()
	gen_parse()
	gen_struct()
}

function gen_regfunc() {
	print ""
	print "static void register_msg(void) __attribute__((__constructor__));"
	print "static void"
	print "register_msg(void)"
	print "{"
	for(i = 0; i < mcnt; i++) {
		print "\tunitcl_register_msg(UNI_" toupper(messages[i]) ", &msgdecl_" messages[i]");"
	}
	print "}"
}

function gen_fmt() {
	print ""
	print "int"
	print "fmt_msg_" msg "(Tcl_Interp *interp, Tcl_DString *str, const struct uni_" msg " *msg)"
	print "{"
	print "\tint ret = TCL_OK;"
	if(msgrep) {
		print "\tu_int i;"
	}
	print ""
	for(i = 0; i < cnt; i++) {
		ie = iename[i]
		if(ienum[i] == "-") {
			print "\tif(fmt_" ie "(interp, str, &msg->" ie ") != TCL_OK)"
			print "\t\tret = TCL_ERROR;"
		} else {
			if(ierep[i]) {
				print "\tif(fmt_repeat(interp, str, &msg->" ie "_repeat) != TCL_OK)"
				print "\t\tret = TCL_ERROR;"
			}
	  		print "\tfor(i = 0; i < " ienum[i] "; i++)"
			print "\t\tif(fmt_" ie "(interp, str, &msg->" ie "[i]) != TCL_OK)"
			print "\t\t\tret = TCL_ERROR;"
		}
	}
	print ""
	print "\treturn ret;"
	print "}"
}

function gen_parse() {
	print ""
	print "int"
	print "parse_msg_" msg "(Tcl_Interp *interp, int argc, const char **argv,"
	print "struct uni_" msg " *out, struct unicx *cx)"
	print "{"
	print "	enum uni_ietype type;"
	print "	union uni_ieall ie;"
	print "	int opt;"
	if (msgrep) {
		print "\tu_int i;"
	}
	if (msgrepie) {
		print "\tstruct uni_ie_repeat repeat;"
	}
	print ""
	print "	cx = cx;"
	if (msgrepie) {
		print "	memset(&repeat, 0, sizeof(repeat));"
	}
	print "	for (opt = 0; opt < argc; opt++) {"
	print "		if (parse_ie(interp, argv[opt], &ie, &type))"
	print "			return (TCL_ERROR);"
	print ""
	print "		switch(type) {"
	print ""
	print "		  default:"
	print "			goto badie;"
	print "		  badie:"
	print "			return (unitcl_setres(interp,"
	print "			    \"" msg ": illegal IE\"));"

	for(i = 0; i < cnt; i++) {
		ie = iename[i]
		print ""
		print "\t\t  case UNI_IE_" toupper(ie) ":"
		if(ierep[i]) {
			print "\t\t\tif(IE_ISPRESENT(repeat)) {"
			print "\t\t\t\tif(IE_ISPRESENT(out->" ie "_repeat))"
			print "\t\t\t\t\treturn (unitcl_setres(interp, \"" msg "." ie "_repeat: already present\"));"
			print "\t\t\t\tout->" ie "_repeat = repeat;"
			print "\t\t\t\trepeat.h.present = 0;"
			print "\t\t\t}"
		}
		if(ienum[i] == "-") {
			print "\t\t\tif(IE_ISPRESENT(out->" ie "))"
			print "\t\t\t\treturn (unitcl_setres(interp, \"" msg "." ie ": already present\"));"
			print "\t\t\tout->" ie " = ie." ie ";"
		} else {
			print "\t\t\tfor(i = 0; i < " ienum[i]" ; i++)"
			print "\t\t\t\tif(!IE_ISPRESENT(out->" ie "[i])) {"
			print "\t\t\t\t\tout->" ie "[i] = ie." ie ";"
			print "\t\t\t\t\tbreak;"
			print "\t\t\t\t}"
			print "\t\t\tif(i == " ienum[i]" )"
			print "\t\t\t\treturn (unitcl_setres(interp, \"" msg "." ie ": too many of them\"));"
		}
		print "\t\t\tbreak;"
	}
	if(msgrepie) {
		print ""
		print "\t\t  case UNI_IE_REPEAT:"
		print "\t\t\tif(IE_ISPRESENT(repeat))"
		print "\t\t\t\treturn (unitcl_setres(interp, \"cannot repeat repeat IE\"));"
		print "\t\t\trepeat = ie.repeat;"
		print "\t\t\tbreak;"
	}
	print ""
	print "\t\t}"
	if(msgrepie) {
		print "\t\tif(IE_ISPRESENT(repeat) && type != UNI_IE_REPEAT)"
		print "\t\t\treturn (unitcl_setres(interp, \"dangling repeat IE\"));"
	}
	print "\t}"
	print "\treturn TCL_OK;"
	print "}"
}

function gen_struct() {
	print ""
	print "static const struct msgdecl msgdecl_" msg " = {"
	print "\t0,"
	print "\t\"" msg "\","
	print "\t(msg_fmt_f)fmt_msg_" msg ","
	print "\t(msg_parse_f)parse_msg_" msg
	print "};"
}
