/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_aal.c,v 1.10 2004/05/14 09:26:53 novo Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_AAL, aal, 0)

/*
 * {aal aal0}
 * {aal aal1 <type> <cbr_rate> {mult <mult>} {screc <screc>} {ecm <ecm>}
 *			       {bsize <bsize>} {part <part>}}
 * {aal aal2}
 * {aal aal4 {cpcs <fwd> <bwd>} {mid <low> <high>} {sscs <sscs>}}
 * {aal aal5 {cpcs <fwd> <bwd>} {sscs <sscs>}}
 * {aal aalu <byte1> <byte2> <byte3> <byte4>}
 */
int
parse_aal(Tcl_Interp *interp, int argc, const char *argv[], union uni_ieall *ie)
{
	u_int n;
	int i, ac;
	const char **av;

	if (argc < 1)
		return (unitcl_setres(interp, "not enough args for aal"));

	if (strcmp(argv[0], "aal0") == 0) {
		if (argc != 1)
			return (unitcl_setres(interp,
			    "bad no of args for aal0"));
		ie->aal.type = UNI_AAL_1;

	} else if (strcmp(argv[0], "aal1") == 0) {
		if (argc < 3)
			return (unitcl_setres(interp,
			    "not enough args for aal1"));
		ie->aal.type = UNI_AAL_1;
		if (unitcl_parse_num(interp, argv[1], &n))
			return (TCL_ERROR);
		ie->aal.u.aal1.subtype = n;
		if (unitcl_parse_num(interp, argv[2], &n))
			return (TCL_ERROR);
		ie->aal.u.aal1.cbr_rate = n;
		for (i = 3; i < argc; i++) {
			if (Tcl_SplitList(interp, argv[i], &ac, &av) != TCL_OK)
				return (TCL_ERROR);
			if (strcmp(av[0], "mult") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal1.mult"));
				}
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.h.present |= UNI_AAL1_MULT_P;
				ie->aal.u.aal1.mult = n;

			} else if (strcmp(av[0], "screc") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal1.screc"));
				}
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.h.present |= UNI_AAL1_SCREC_P;
				ie->aal.u.aal1.screc = n;

			} else if (strcmp(av[0], "ecm") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal1.ecm"));
				}
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.h.present |= UNI_AAL1_ECM_P;
				ie->aal.u.aal1.ecm = n;

			} else if (strcmp(av[0], "bsize") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal1.bsize"));
				}
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.h.present |= UNI_AAL1_BSIZE_P;
				ie->aal.u.aal1.bsize = n;

			} else if (strcmp(av[0], "part") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal1.part"));
				}
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.h.present |= UNI_AAL1_PART_P;
				ie->aal.u.aal1.part = n;

			} else {
				Tcl_Free((char *)av);
				return (unitcl_setres(interp,
				    "bad key for aal1"));
			}

			Tcl_Free((char *)av);
		}

	} else if (strcmp(argv[0], "aal2") == 0) {
		if (argc != 1)
			return (unitcl_setres(interp,
			    "bad no of args for aal2"));

		ie->aal.type = UNI_AAL_2;

	} else if (strcmp(argv[0], "aal4") == 0) {
		ie->aal.type = UNI_AAL_4;
		for (i = 1; i < argc; i++) {
			if (Tcl_SplitList(interp, argv[i], &ac, &av) != TCL_OK)
				return (TCL_ERROR);
			if (strcmp(av[0], "cpcs") == 0) {
				if (ac != 3) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal4.cpcs"));
				}
				ie->aal.h.present |= UNI_AAL4_CPCS_P;
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal4.fwd_cpcs = n;
				if(unitcl_parse_num(interp, av[2], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal4.bwd_cpcs = n;

			} else if (strcmp(av[0], "mid") == 0) {
				if (ac != 3) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal4.mid"));
				}
				ie->aal.h.present |= UNI_AAL4_MID_P;
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal4.mid_low = n;
				if (unitcl_parse_num(interp, av[2], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal4.mid_high = n;

			} else if (strcmp(av[0], "sscs") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal4.sscs"));
				}
				ie->aal.h.present |= UNI_AAL4_SSCS_P;
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal4.sscs = n;

			} else {
				Tcl_Free((char *)av);
				return (unitcl_setres(interp,
				    "bad key for aal4"));
			}

			Tcl_Free((char *)av);
		}

	} else if (strcmp(argv[0], "aal5") == 0) {
		ie->aal.type = UNI_AAL_5;
		for (i = 1; i < argc; i++) {
			if (Tcl_SplitList(interp, argv[i], &ac, &av) != TCL_OK)
				return (TCL_ERROR);
			if (strcmp(av[0], "cpcs") == 0) {
				if (ac != 3) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal5.cpcs"));
				}
				ie->aal.h.present |= UNI_AAL5_CPCS_P;
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal5.fwd_cpcs = n;
				if (unitcl_parse_num(interp, av[2], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal5.bwd_cpcs = n;

			} else if (strcmp(av[0], "sscs") == 0) {
				if (ac != 2) {
					Tcl_Free((char *)av);
					return (unitcl_setres(interp,
					    "bad no of args for aal5.sscs"));
				}
				ie->aal.h.present |= UNI_AAL5_SSCS_P;
				if (unitcl_parse_num(interp, av[1], &n)) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				ie->aal.u.aal5.sscs = n;

			} else {
				Tcl_Free((char *)av);
				return (unitcl_setres(interp,
				    "bad key for aal5"));
			}

			Tcl_Free((char *)av);
		}

	} else if (strcmp(argv[0], "aalu") == 0) {
		ie->aal.type = UNI_AAL_USER;
		if (argc > 5)
			return (unitcl_setres(interp,
			    "aal_user needs lesser than 4 arguments"));
		for (i = 1; i < argc; i++) {
			if (unitcl_parse_num(interp, argv[i], &n))
				return (TCL_ERROR);
			ie->aal.u.aalu.user[ie->aal.u.aalu.len++] = n;
		}

	} else
		return (unitcl_setres(interp, "bad aal type"));

	IE_ADDPRESENT(ie->aal);

	return (TCL_OK);
}

FMT_IE_START(UNI_IE_AAL, aal)
{
	char buf[100];
	u_int i;

	if (ie->type == UNI_AAL_0) {
		Tcl_DStringAppendElement(str, "aal0");

	} else if (ie->type == UNI_AAL_1) {
		Tcl_DStringAppendElement(str, "aal1");

		sprintf(buf, "%u", ie->u.aal1.subtype);
		Tcl_DStringAppendElement(str, buf);

		sprintf(buf, "%u", ie->u.aal1.cbr_rate);
		Tcl_DStringAppendElement(str, buf);

		if (ie->h.present & UNI_AAL1_MULT_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "mult");

			sprintf(buf, "%u", ie->u.aal1.mult);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL1_SCREC_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "screc");

			sprintf(buf, "%u", ie->u.aal1.screc);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL1_ECM_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "ecm");

			sprintf(buf, "%u", ie->u.aal1.ecm);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL1_BSIZE_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "bsize");

			sprintf(buf, "%u", ie->u.aal1.bsize);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL1_PART_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "part");

			sprintf(buf, "%u", ie->u.aal1.part);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

	} else if (ie->type == UNI_AAL_2) {
		Tcl_DStringAppendElement(str, "aal2");

	} else if (ie->type == UNI_AAL_4) {
		Tcl_DStringAppendElement(str, "aal4");

		if (ie->h.present & UNI_AAL4_CPCS_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "cpcs");

			sprintf(buf, "%u", ie->u.aal4.fwd_cpcs);
			Tcl_DStringAppendElement(str, buf);

			sprintf(buf, "%u", ie->u.aal4.bwd_cpcs);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL4_MID_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "mid");

			sprintf(buf, "%u", ie->u.aal4.mid_low);
			Tcl_DStringAppendElement(str, buf);

			sprintf(buf, "%u", ie->u.aal4.mid_high);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL4_SSCS_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "sscs");

			sprintf(buf, "%u", ie->u.aal4.sscs);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

	} else if (ie->type == UNI_AAL_5) {
		Tcl_DStringAppendElement(str, "aal5");

		if (ie->h.present & UNI_AAL5_CPCS_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "cpcs");

			sprintf(buf, "%u", ie->u.aal5.fwd_cpcs);
			Tcl_DStringAppendElement(str, buf);

			sprintf(buf, "%u", ie->u.aal5.bwd_cpcs);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

		if (ie->h.present & UNI_AAL5_SSCS_P) {
			Tcl_DStringStartSublist(str);
			Tcl_DStringAppendElement(str, "sscs");

			sprintf(buf, "%u", ie->u.aal5.sscs);
			Tcl_DStringAppendElement(str, buf);

			Tcl_DStringEndSublist(str);
		}

	} else if (ie->type == UNI_AAL_USER) {
		Tcl_DStringAppendElement(str, "aalu");

		for (i = 0; i < ie->u.aalu.len; i++) {
			sprintf(buf, "0x%02x", ie->u.aalu.user[i]);
			Tcl_DStringAppendElement(str, buf);
		}
	} else
		return (unitcl_setres(interp, "bad all type %u", ie->type));
}
FMT_IE_END()
