/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_callstate.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_CALLSTATE, callstate, 0)

/*
 * {callstate state}
 */
int
parse_callstate(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc != 1) {
		unitcl_setres(interp, "bad # of args for callstate");
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[0], &n))
		return TCL_ERROR;

	switch(n) {
	  case UNI_CALLSTATE_U0:
	  case UNI_CALLSTATE_U1:
	  case UNI_CALLSTATE_U3:
	  case UNI_CALLSTATE_U4:
	  case UNI_CALLSTATE_U6:
	  case UNI_CALLSTATE_U7:
	  case UNI_CALLSTATE_U8:
	  case UNI_CALLSTATE_U9:
	  case UNI_CALLSTATE_U10:
	  case UNI_CALLSTATE_U11:
	  case UNI_CALLSTATE_U12:
	  case UNI_CALLSTATE_REST1:
	  case UNI_CALLSTATE_REST2:
	  case UNI_CALLSTATE_U13:
	  case UNI_CALLSTATE_U14:
		break;

	  default:
		unitcl_setres(interp, "bad callstate");
		return TCL_ERROR;
	}
	ie->callstate.state = n;

	IE_ADDPRESENT(ie->callstate);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_CALLSTATE, callstate)
{
	char buf[100];

	sprintf(buf, "%u", ie->state);
	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()
