/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_connid.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_CONNID, connid, 0)

/*
 * {connid a/n <vpi> [<vci>|any]}
 */
int
parse_connid(Tcl_Interp *interp, int argc, const char **argv, union uni_ieall *ie)
{
	u_int n;

	if(argc == 1 || argc > 3) {
		unitcl_setres(interp, "connid: bad # of args");
		return TCL_ERROR;
	}
	if(strcmp(argv[0], "a") == 0)
		ie->connid.assoc = UNI_CONNID_ASSOC;
	else if(strcmp(argv[0], "n") == 0)
		ie->connid.assoc = UNI_CONNID_NONASSOC;
	else {
		unitcl_setres(interp, "connid: bad assoc flag");
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[1], &n))
		return TCL_ERROR;
	ie->connid.vpci = n;

	if(argc == 2) {
		ie->connid.type = UNI_CONNID_NOVCI;

	} else if(argc == 3) {
		if(strcmp(argv[2], "any") == 0) {
			ie->connid.type = UNI_CONNID_ANYVCI;
		} else {
			ie->connid.type = UNI_CONNID_VCI;
			if(unitcl_parse_num(interp, argv[2], &n))
				return TCL_ERROR;
			ie->connid.vci = n;
		}
	}

	IE_ADDPRESENT(ie->connid);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_CONNID, connid)
{
	char buf[100];

	if(ie->assoc == UNI_CONNID_ASSOC)
		Tcl_DStringAppendElement(str, "a");
	else
		Tcl_DStringAppendElement(str, "n");
	sprintf(buf, "%u", ie->vpci);
	Tcl_DStringAppendElement(str, buf);
	if(ie->type == UNI_CONNID_ANYVCI)
		Tcl_DStringAppendElement(str, "any");
	else if(ie->type == UNI_CONNID_VCI) {
		sprintf(buf, "%u", ie->vci);
		Tcl_DStringAppendElement(str, buf);
	}
}
FMT_IE_END()
