/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_cscope.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_CSCOPE, cscope, 0)

/*
 * {cscope <type> <scope>}
 * type := org
 * scope := NUM
 */
int
parse_cscope(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc != 2) {
		unitcl_setres(interp, "bad number of # to cscope");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "org") == 0)
		ie->cscope.type = UNI_CSCOPE_ORG;
	else {
		unitcl_setres(interp, "bad cscope type '%s'", argv[0]);
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[1], &n) != TCL_OK)
		return TCL_ERROR;
	ie->cscope.scope = n;

	IE_ADDPRESENT(ie->cscope);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_CSCOPE, cscope)
{
	char buf[100];

	switch(ie->type) {

	  default:
		unitcl_setres(interp, "bad cscope type to format");
		return TCL_ERROR;

	  case UNI_CSCOPE_ORG:
		Tcl_DStringAppendElement(str, "org");
		break;
	}
	sprintf(buf, "%u", ie->scope);
	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()
