/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_dtl.c,v 1.9 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_DTL, dtl, 0)

/*
 * {dtl <ptr> [{<level> <id> <port>}]*}
 */
int
parse_dtl(Tcl_Interp *interp, int argc, const char **argv, union uni_ieall *ie)
{
	u_int n;
	int i, ac;
	const char **av;

	if(argc < 1) {
		unitcl_setres(interp, "not enough args for dtl");
		return TCL_ERROR;
	}
	if(argc >= UNI_DTL_MAXNUM + 1) {
		unitcl_setres(interp, "too many dtl elements (max is %d)", UNI_DTL_MAXNUM);
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[0], &n))
		return TCL_ERROR;
	if(n >= (1<<16)) {
		unitcl_setres(interp, "dtl pointer too larger for 16 bit");
		return TCL_ERROR;
	}
	if(n % 27 != 0) {
		unitcl_setres(interp, "dtl pointer must be multiple of 27");
		return TCL_ERROR;
	}
	ie->dtl.ptr = n;

	ie->dtl.num = 0;
	for(i = 1; i < argc; i++) {
		if(Tcl_SplitList(interp, argv[i], &ac, &av) != TCL_OK)
			return TCL_ERROR;

		if(ac != 3) {
			unitcl_setres(interp, "need 3 elements in dtl element");
			goto err;
		}
		if(unitcl_parse_nodeid(interp, (const char **)av,
			&ie->dtl.dtl[ie->dtl.num].node_level,
			ie->dtl.dtl[ie->dtl.num].node_id) != TCL_OK)
			goto err;

		if(unitcl_parse_num(interp, av[2], &n))
			goto err;
		ie->dtl.dtl[ie->dtl.num].port_id = n;

		ie->dtl.num++;

		Tcl_Free((char *)av);
	}

	IE_ADDPRESENT(ie->dtl);
	return TCL_OK;
err:
	Tcl_Free((char *)av);
	return TCL_ERROR;
}


FMT_IE_START(UNI_IE_DTL, dtl)
{
	char buf[100];
	u_int i, j;

	sprintf(buf, "%u", ie->ptr);
	Tcl_DStringAppendElement(str, buf);

	for(i = 0; i < ie->num; i++) {
		Tcl_DStringStartSublist(str);

		sprintf(buf, "%d", ie->dtl[i].node_level);
		Tcl_DStringAppendElement(str, buf);

		for(j = 0; j < 21; j++)
			sprintf(buf+2*j, "%02x", ie->dtl[i].node_id[j]);
		Tcl_DStringAppendElement(str, buf);

		sprintf(buf, "0x%x", ie->dtl[i].port_id);
		Tcl_DStringAppendElement(str, buf);

		Tcl_DStringEndSublist(str);
	}
}
FMT_IE_END()
