/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_eetd.c,v 1.9 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_EETD, eetd, 0)

/*
 * {eetd <cum> [<max>] [net]}
 */
int
parse_eetd(Tcl_Interp *interp, int argc, const char **argv, union uni_ieall *ie)
{

	if (argc < 1 || argc > 3)
		return (unitcl_setres(interp, "bad # of args for eetd"));

	if (unitcl_parse_num(interp, argv[0], &ie->eetd.cumulative) != TCL_OK)
		return (TCL_ERROR);
	ie->eetd.h.present |= UNI_EETD_CUM_P;

	if (argc > 1) {
		if (strcmp(argv[1], "any") == 0) {
			ie->eetd.maximum = UNI_EETD_ANYMAX;
		} else if (unitcl_parse_num(interp, argv[1], &ie->eetd.maximum)
		    != TCL_OK)
			return (TCL_ERROR);
		ie->eetd.h.present |= UNI_EETD_MAX_P;

		if (argc > 2) {
			if (strcmp(argv[1], "net") != 0)
				return (unitcl_setres(interp,
				    "bad indicator in eetd"));
			ie->eetd.h.present |= UNI_EETD_NET_P;
		}
	}

	IE_ADDPRESENT(ie->eetd);

	return (TCL_OK);
}

FMT_IE_START(UNI_IE_EETD, eetd)
{
	char buf[100];

	sprintf(buf, "%u", ie->cumulative);
	Tcl_DStringAppendElement(str, buf);

	if (ie->h.present & (UNI_EETD_MAX_P | UNI_EETD_NET_P)) {
		if (!(ie->h.present & UNI_EETD_MAX_P) ||
		    ie->maximum == UNI_EETD_ANYMAX)
			Tcl_DStringAppendElement(str, "any");
		else {
			sprintf(buf, "%u", ie->maximum);
			Tcl_DStringAppendElement(str, buf);
		}
		if (ie->h.present & UNI_EETD_NET_P)
			Tcl_DStringAppendElement(str, "net");
	}
}
FMT_IE_END()
