/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_epref.c,v 1.9 2004/01/02 15:15:55 hbb Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_EPREF, epref, 0)

/*
 * {epref <from>|<to> <epref>}
 */
int
parse_epref(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if (argc != 2)
		return (unitcl_setres(interp, "epref needs two arguments"));

	if (strcmp(argv[0], "from") == 0 || strcmp(argv[0], "0") == 0)
		ie->epref.flag = 0;
	else if (strcmp(argv[0], "to") == 0 || strcmp(argv[0], "1") == 0)
		ie->epref.flag = 1;
	else
		return (unitcl_setres(interp, "bad flag for epref"));

	if (unitcl_parse_num(interp, argv[1], &n))
		return (TCL_ERROR);
	ie->epref.epref = n;

	IE_ADDPRESENT(ie->epref);

	return (TCL_OK);
}

FMT_IE_START(UNI_IE_EPREF, epref)
{
	char buf[100];

	if (ie->flag)
		Tcl_DStringAppendElement(str, "1");
	else
		Tcl_DStringAppendElement(str, "0");
	sprintf(buf, "%u", ie->epref);
	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()
