/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_exqos.c,v 1.9 2004/05/12 11:50:04 novo Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_EXQOS, exqos, 0)

/*
 * {exqos {<type> <val>}*}
 * type := facc | bacc | fcum | bcum | fclr | bclr
 */
static const struct toff {
	const char	*name;
	size_t		off;
	u_int		flag;
	u_int		any;
} offtab[] = {
  {"facc", offsetof(struct uni_ie_exqos, facc), UNI_EXQOS_FACC_P, UNI_EXQOS_ANY_CDV },
  {"bacc", offsetof(struct uni_ie_exqos, bacc), UNI_EXQOS_BACC_P, UNI_EXQOS_ANY_CDV },
  {"fcum", offsetof(struct uni_ie_exqos, fcum), UNI_EXQOS_FCUM_P, 0xffffffff },
  {"bcum", offsetof(struct uni_ie_exqos, bcum), UNI_EXQOS_BCUM_P, 0xffffffff },
  {"fclr", offsetof(struct uni_ie_exqos, fclr), UNI_EXQOS_FCLR_P, UNI_EXQOS_ANY_CLR },
  {"bclr", offsetof(struct uni_ie_exqos, bclr), UNI_EXQOS_BCLR_P, UNI_EXQOS_ANY_CLR },
  { NULL, 0, 0, 0 }
};

int
parse_exqos(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	int ac;
	const char **av;
	u_int i;
	u_int val;

	if (argc == 0)
		return (TCL_OK);

	while (argc--) {
		if (Tcl_SplitList(interp, *argv, &ac, &av) != TCL_OK)
			return (TCL_ERROR);

		if (ac != 2) {
			Tcl_Free((char *)av);
			return (unitcl_setres(interp,
			    "bad # of fields in exqos subfield"));
		}

		for (i = 0; offtab[i].name != NULL; i++)
			if (strcmp(offtab[i].name, av[0]) == 0) {
				if (offtab[i].any != 0xffffffff &&
				    strcmp(av[1], "any") == 0)
					val = offtab[i].any;
				else if (unitcl_parse_num(interp, av[1], &val)
				    != TCL_OK) {
					Tcl_Free((char *)av);
					return (TCL_ERROR);
				}
				*(u_int *)(void *)((u_char *)&ie->exqos +
				    offtab[i].off) = val;
				ie->exqos.h.present |= offtab[i].flag;
				break;
			}
		if (offtab[i].name == NULL) {
			Tcl_Free((char *)av);
			return (unitcl_setres(interp, "bad exqos subfield"));
		}
		Tcl_Free((char *)av);
		argv++;
	}

	IE_ADDPRESENT(ie->exqos);

	return (TCL_OK);
}

FMT_IE_START(UNI_IE_EXQOS, exqos)
{
	char buf[100];

#define DOFMT(BIG,SMALL,ANY)						\
	if (ie->h.present & UNI_EXQOS_##BIG##_P) {			\
		Tcl_DStringStartSublist(str);				\
		Tcl_DStringAppendElement(str, #SMALL);			\
		if (ie->SMALL == ANY)					\
			Tcl_DStringAppendElement(str, "any");		\
		else {							\
			sprintf(buf, "%d", ie->SMALL);			\
			Tcl_DStringAppendElement(str, buf);		\
		}							\
		Tcl_DStringEndSublist(str);				\
	}
	DOFMT(FACC, facc, UNI_EXQOS_ANY_CDV);
	DOFMT(BACC, bacc, UNI_EXQOS_ANY_CDV);
	DOFMT(FCUM, fcum, 0xffffffff);
	DOFMT(BCUM, bcum, 0xffffffff);
	DOFMT(FCLR, fclr, UNI_EXQOS_ANY_CLR);
	DOFMT(BCLR, bclr, UNI_EXQOS_ANY_CLR);
#undef DOFMT
}
FMT_IE_END()
