/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_lij.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_LIJ_CALLID, lij_callid, 0)
REGISTER_IE(UNI_IE_LIJ_PARAM, lij_param, 0)
REGISTER_IE(UNI_IE_LIJ_SEQNO, lij_seqno, 0)

/*
 * {lij_callid <type> <id>}
 * type := root
 *
 * {lij_param <screen>}
 * screen := net
 *
 * {lij_seqno <seqno>}
 */
int
parse_lij_callid(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	if(argc != 2) {
		unitcl_setres(interp, "bad # of args for lij_callid");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "root") == 0)
		ie->lij_callid.type = UNI_LIJ_IDTYPE_ROOT;
	else {
		unitcl_setres(interp, "bad lij_callid type");
		return TCL_ERROR;
	}
	if(unitcl_parse_num(interp, argv[1], &ie->lij_callid.callid) != TCL_OK)
		return TCL_ERROR;
	IE_ADDPRESENT(ie->lij_callid);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_LIJ_CALLID, lij_callid)
{
	char buf[100];

	switch(ie->type) {

	  default:
		unitcl_setres(interp, "bad callid type to format");
		return TCL_ERROR;

	  case UNI_LIJ_IDTYPE_ROOT:
		Tcl_DStringAppendElement(str, "root");
		break;
	}

	sprintf(buf, "%u", ie->callid);
	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()

/**********************************************************************/

int
parse_lij_param(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	if(argc != 1) {
		unitcl_setres(interp, "bad # of args for lij_param");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "net") == 0)
		ie->lij_param.screen = UNI_LIJ_SCREEN_NETJOIN;
	else {
		unitcl_setres(interp, "bad lij_param screen indicator");
		return TCL_ERROR;
	}
	IE_ADDPRESENT(ie->lij_param);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_LIJ_PARAM, lij_param)
{
	switch(ie->screen) {

	  default:
		unitcl_setres(interp, "bad screen indicator to format");
		return TCL_ERROR;

	  case UNI_LIJ_SCREEN_NETJOIN:
		Tcl_DStringAppendElement(str, "net");
		break;
	}
}
FMT_IE_END()

/**********************************************************************/

int
parse_lij_seqno(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	if(argc != 1) {
		unitcl_setres(interp, "bad # of args for lij_seqno");
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[0], &ie->lij_seqno.seqno) != TCL_OK)
		return TCL_ERROR;

	IE_ADDPRESENT(ie->lij_seqno);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_LIJ_SEQNO, lij_seqno)
{
	char buf[100];

	sprintf(buf, "%u", ie->seqno);
	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()
