/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_notify.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_NOTIFY, notify, 0)

/*
 * {notify [byte ...]}
 */
int
parse_notify(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc == 0)
		return TCL_OK;

	while(argc--) {
		if(ie->notify.len >= UNI_NOTIFY_MAXLEN) {
			unitcl_setres(interp, "too many notify info");
			return TCL_ERROR;
		}
		if(unitcl_parse_num(interp, *argv++, &n) != TCL_OK)
			return TCL_ERROR;
		ie->notify.notify[ie->notify.len++] = n;
	}

	IE_ADDPRESENT(ie->notify);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_NOTIFY, notify)
{
	char buf[100];
	u_int i;

	for(i = 0; i < ie->len; i++) {
		sprintf(buf, "0x%02x", ie->notify[i]);
		Tcl_DStringAppendElement(str, buf);
	}
}
FMT_IE_END()
