/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_restart.c,v 1.7 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_RESTART, restart, 0)

/*
 * {restart class}
 */
int
parse_restart(Tcl_Interp *interp, int argc, const char **argv, union uni_ieall *ie)
{
	if(argc != 1) {
		unitcl_setres(interp, "bad # of args for restart");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "channel") == 0)
		ie->restart.rclass = UNI_RESTART_CHANNEL;

	else if(strcmp(argv[0], "all") == 0)
		ie->restart.rclass = UNI_RESTART_ALL;

	else if(strcmp(argv[0], "path") == 0)
		ie->restart.rclass = UNI_RESTART_PATH;

	else {
		unitcl_setres(interp, "bad restart class");
		return TCL_ERROR;
	}

	IE_ADDPRESENT(ie->restart);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_RESTART, restart)
{
	if(ie->rclass == UNI_RESTART_CHANNEL)
		Tcl_DStringAppendElement(str, "channel");
	else if(ie->rclass == UNI_RESTART_ALL)
		Tcl_DStringAppendElement(str, "all");
	else if(ie->rclass == UNI_RESTART_PATH)
		Tcl_DStringAppendElement(str, "path");
	else {
		unitcl_setres(interp, "bad restart class %d", ie->rclass);
		return TCL_ERROR;
	}
}
FMT_IE_END()
