/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_soft.c,v 1.8 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_CALLED_SOFT, called_soft, 0)
REGISTER_IE(UNI_IE_CALLING_SOFT, calling_soft, 0)

/*
 * {called_soft <type> [vpi] [vci]}
 * type := any|req|ass
 */
int
parse_called_soft(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc < 1) {
		unitcl_setres(interp, "not enough args for called_soft");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "any") == 0)
		ie->called_soft.sel = UNI_SOFT_SEL_ANY;
	else if(strcmp(argv[0], "req") == 0)
		ie->called_soft.sel = UNI_SOFT_SEL_REQ;
	else if(strcmp(argv[0], "ass") == 0)
		ie->called_soft.sel = UNI_SOFT_SEL_ASS;
	else {
		unitcl_setres(interp, "%s: bad called_soft selection", argv[0]);
		return TCL_ERROR;
	}

	if(argc > 1) {
		if(unitcl_parse_num(interp, argv[1], &n))
			return TCL_ERROR;
		ie->called_soft.vpi = n;
		ie->called_soft.h.present |= UNI_CALLED_SOFT_VPI_P;
	}
	if(argc > 2) {
		if(unitcl_parse_num(interp, argv[2], &n))
			return TCL_ERROR;
		ie->called_soft.vci = n;
		ie->called_soft.h.present |= UNI_CALLED_SOFT_VCI_P;
	}

	IE_ADDPRESENT(ie->called_soft);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_CALLED_SOFT, called_soft)
{
	char buf[100];

	switch(ie->sel) {

	  case UNI_SOFT_SEL_ANY:
		Tcl_DStringAppendElement(str, "any");
		break;

	  case UNI_SOFT_SEL_REQ:
		Tcl_DStringAppendElement(str, "req");
		break;

	  case UNI_SOFT_SEL_ASS:
		Tcl_DStringAppendElement(str, "ass");
		break;

	  default:
		unitcl_setres(interp, "bad called_soft selection: %d", ie->sel);
		return TCL_ERROR;
	}

	if(ie->h.present & UNI_CALLED_SOFT_VPI_P) {
		sprintf(buf, "%u", ie->vpi);
		Tcl_DStringAppendElement(str, buf);

		if(ie->h.present & UNI_CALLED_SOFT_VCI_P) {
			sprintf(buf, "%u", ie->vci);
			Tcl_DStringAppendElement(str, buf);
		}
	}

}
FMT_IE_END()

/*
 * {calling_soft vpi [vci]}
 */
int
parse_calling_soft(Tcl_Interp *interp, int argc, const char **argv,
   union uni_ieall *ie)
{
	u_int n;

	if(argc < 1) {
		unitcl_setres(interp, "not enough args for calling_soft");
		return TCL_ERROR;
	}

	if(unitcl_parse_num(interp, argv[0], &n))
		return TCL_ERROR;
	ie->calling_soft.vpi = n;

	if(argc > 1) {
		if(unitcl_parse_num(interp, argv[1], &n))
			return TCL_ERROR;
		ie->calling_soft.vci = n;
		ie->calling_soft.h.present |= UNI_CALLING_SOFT_VCI_P;
	}

	IE_ADDPRESENT(ie->calling_soft);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_CALLING_SOFT, calling_soft)
{
	char buf[100];

	sprintf(buf, "%u", ie->vpi);
	Tcl_DStringAppendElement(str, buf);

	if(ie->h.present & UNI_CALLING_SOFT_VCI_P) {
		sprintf(buf, "%u", ie->vci);
		Tcl_DStringAppendElement(str, buf);
	}

}
FMT_IE_END()
