/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_tns.c,v 1.7 2004/01/02 15:15:55 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_TNS, tns, 0)

/*
 * {tns <TNS>}
 */
int
parse_tns(Tcl_Interp *interp, int argc, const char **argv, union uni_ieall *ie)
{
	const char *ptr;

	if(argc != 1) {
		unitcl_setres(interp, "tns needs one arg");
		return TCL_ERROR;
	}

	ptr = argv[0];
	if(*ptr != '"') {
		unitcl_setres(interp, "bad TNS string");
		return TCL_ERROR;
	}
	ptr++;
	if(strlen(ptr) > sizeof(ie->tns.net) + 1 || strlen(ptr) == 0) {
		unitcl_setres(interp, "TNS too long");
		return TCL_ERROR;
	}
	ie->tns.len = strlen(ptr) - 1;
	(void)memcpy(ie->tns.net, ptr, ie->tns.len);
	ptr += ie->tns.len;
	if(*ptr != '"') {
		unitcl_setres(interp, "bad TNS string");
		return TCL_ERROR;
	}

	IE_ADDPRESENT(ie->tns);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_TNS, tns)
{
	char buf[100], *p;
	u_int i;

	p = buf;
	*p++ = '"';
	for(i = 0; i < ie->len; i++) {
		if(!isascii(ie->net[i]) || !isprint(ie->net[i]))
			p += sprintf(p, "\\%03o", ie->net[i]);
		else
			*p++ = ie->net[i];
	}
	*p++ = '"';
	*p++ = '\0';

	Tcl_DStringAppendElement(str, buf);
}
FMT_IE_END()
