/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/unitcl_priv.h,v 1.7 2004/07/08 13:33:44 brandt Exp $
 */
#ifndef unitcl_priv_h_
#define unitcl_priv_h_

struct iedecl {
	u_int	flags;
	const char *name;
	int	(*parse)(Tcl_Interp *, int, const char **, union uni_ieall *);
};

typedef int (*msg_parse_f)(Tcl_Interp *, int, const char **, union uni_msgall *, struct unicx *);
typedef int (*msg_fmt_f)(Tcl_Interp *, Tcl_DString *, const union uni_msgall *);

struct msgdecl {
	u_int	flags;
	const char *name;
	msg_fmt_f fmt;
	msg_parse_f parse;
};

int unitcl_parse_num(Tcl_Interp *, const char *, u_int *);
int unitcl_parse_nodeid(Tcl_Interp *, const char **, u_char *, u_char *);
int unitcl_fmt_iehdr(Tcl_Interp *, Tcl_DString *, u_int,
	const struct uni_iehdr *);
void unitcl_register_ie(u_int, const struct iedecl *);
void unitcl_register_msg(u_int, const struct msgdecl *);

#define REGISTER_IE(CODE,FIELD,FLAGS) 					\
	static const struct iedecl iedecl_##FIELD = {			\
		FLAGS, #FIELD,						\
		parse_##FIELD						\
	};								\
	static void register_##FIELD(void)				\
	    __attribute__((__constructor__));				\
	static void register_##FIELD(void)				\
	{								\
		unitcl_register_ie(CODE, &iedecl_##FIELD);		\
	}

#define REGISTER_MSG(CODE,FIELD,FLAGS) 					\
	int fmt_msg_##FIELD(Tcl_Interp *, Tcl_DString *,		\
	    struct uni_##FIELD *);					\
	int parse_msg_##FIELD(Tcl_Interp *, int, char *[],		\
	    struct uni_##FIELD *, struct unicx *);			\
	static const struct msgdecl msgdecl_##FIELD = { FLAGS, #FIELD,	\
		(msg_fmt_f)fmt_msg_##FIELD,				\
		(msg_parse_f)parse_msg_##FIELD,				\
	};    								\
	static void register_##FIELD()					\
	    __attribute__((__constructor__));				\
	static void register_##FIELD()					\
	{								\
		unitcl_register_msg(CODE, &msgdecl_##FIELD);		\
	}


#define FMT_IE_START(CODE,IE)						\
int									\
fmt_##IE(Tcl_Interp *interp, Tcl_DString *str,				\
    const struct uni_ie_##IE *ie)					\
{									\
	int ret;							\
									\
	if (ie->h.present == 0)						\
		return (TCL_OK);					\
									\
	Tcl_DStringStartSublist(str);					\
	ret = unitcl_fmt_iehdr(interp, str, CODE, &ie->h);		\
	if (ret == TCL_OK) {

#define FMT_IE_END()							\
		Tcl_DStringEndSublist(str);				\
		return (ret);						\
	} else {							\
		Tcl_DStringEndSublist(str);				\
		if (ret == TCL_CONTINUE)				\
			return (TCL_OK);				\
		return (TCL_ERROR);					\
	}								\
}

void unitclbug(const char *fmt, ...) __dead2 __printflike(1, 2)
    __attribute__((__weak__));

#ifdef bsdi
#define	__func__ __FUNCTION__
#endif

#endif
