/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/test_uni/unitest.h,v 1.5 2004/01/02 15:15:56 hbb Exp $
 *
 * UNI test program
 */
#ifndef TEST_UNI_H
#define TEST_UNI_H

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>
#include <netnatm/msg/unimsglib.h>
#include <netnatm/saal/sscopdef.h>
#include <netnatm/saal/sscfudef.h>
#include <netnatm/sig/unidef.h>
#include <netgraph/atm/ng_uni.h>
#include <netgraph/atm/ng_sscop.h>
#include <netgraph/atm/ng_sscfu.h>
#include "unitcl.h"
#include "testlib.h"

struct uni_arg;
struct uni_config;

extern struct unicx *unicx;
extern Tcl_Interp *tcl_interp;

void input_uni(const u_char *, size_t);
void input_man(u_char *, size_t);
void input_dbg(u_char *, size_t);
void input_saal_in(u_char *, size_t);
void input_saal_out(u_char *, size_t);
void input_usr(void);
void try_exec(Tcl_Interp *);
void try_event(Tcl_Interp *, Tcl_DString *);

int cmd_dbg(ClientData, Tcl_Interp *, int, const char *[]);
int cmd_status(ClientData, Tcl_Interp *, int, const char *[]);
int cmd_dosig(ClientData, Tcl_Interp *, int, const char *[]);
int cmd_sleep(ClientData, Tcl_Interp *, int, const char *[]);
int cmd_getconfig(ClientData, Tcl_Interp *, int, const char *[]);

void set_error(Tcl_Interp *, const char *, ...) __printflike(2,3);

void send_control(u_int, void *, size_t);
void send_status(const char *);
void send_uniapi(struct uni_arg *, size_t);
void send_uniapiv(struct uni_arg *, ...);

void do_getconfig(struct uni_config *);
void do_getdbg(struct ngm_uni_debug *);

void init_uni(void);

#endif
