# Copyright (c) 2001-2003
#	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
# 	All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Hartmut Brandt <harti@freebsd.org>
#
# $Begemot: atmsupport/test_uni/unitest.tcl,v 1.1 2003/08/21 11:04:58 hbb Exp $
#
# Author: Harti Brandt <brandt@fokus.gmd.de>
#
set variant 0
set enq 0
set mpt 0
set apvar 0

proc s {v} {
	global variant
	set variant $v
	do_setup
}

proc uniev {ev args} {
	if {$ev == "error"} {
		puts "unierror: $args"
		return
	}
	set cref [lindex $args 0]
	set flag [lindex $args 1]
	set args [lrange $args 2 end]
	global variant

	puts "unievent: $ev $cref $flag $args"

	if {$ev == "setup.indication"} {
		do_enq $cref $flag
		if {$variant == 0 || $variant == 2 || $variant == 3} {
			sleep 1000 "setup_indication $cref"
		}
		if {$variant == 1} {
			dosig release.response $cref 0 default
			do_enq $cref $flag
		}
	}
	if {$ev == "add-party.indication"} {
		set args [lrange $args 1 end]
		sleep 200 "ap_ind $cref $flag $args"
	}
	if {$ev == "drop-party.indication"} {
		set args [lrange $args 1 end]
		sleep 100 "dp_ind $cref $flag $args"
	}
}

proc do_setup {} {
	global mpt

	if {$mpt == 1} {
		dosig setup.request					\
		    {bearer x cbr noclip mp}				\
		    {epref from 0}					\
		    {traffic {f {pcr 1 10}} {b {pcr 1 0}}}		\
		    {qos 0 0}						\
		    {called atme 45000493034637352f0000000000000000000000}

	} else {
		dosig setup.request					\
		    {bearer x cbr noclip p2p}				\
		    {traffic {f {pcr 1 10}} {b {pcr 1 10}}}		\
		    {qos 0 0}						\
		    {called atme 45000493034637352f0000000000000000000000}
	}
}

#
# Got a SETUP indication
#
proc setup_indication {cref} {
	puts "SETUP_INDICATION"
	do_enq $cref 0
	global mpt

	if {$mpt == 1} {
		dosig proceeding.request $cref 0 default {epref to 0}
	} else {
		dosig proceeding.request $cref 0 default
	}
	sleep 1000 "do_alerting $cref"
}
proc do_alerting {cref} {
	puts "DO_ALERTING"
	do_enq $cref 0
	global mpt variant
	if {$mpt == 1} {
		dosig alerting.request $cref 0 default {epref to 0}
	} else {
		dosig alerting.request $cref 0 default
	}
	if {$variant != 3} {
		sleep 1000 "do_connect $cref"
	}
}
proc do_connect {cref} {
	puts "DO_CONNECT"
	global variant mpt
	do_enq $cref 0
	if {$mpt == 1} {
		dosig setup.response $cref 0 default {epref to 0}
	} elseif {$variant == 2} {
		dosig setup.response $cref 0 default {epref to 100}
	} else {
		dosig setup.response $cref 0 default
	}
	do_enq $cref 0
}

proc do_enq {cref flag} {
	global enq

	if {$enq == 0} {return}

	dosig status-enquiry.request $cref $flag
}

proc rr {cref flag} {
	dosig release.response $cref $flag default
}

proc ap {cref} {
	dosig add-party.request $cref 1 default \
	    {called atme 45000493034637352f0000000000000000000000}
}

proc ap_ind {cref flag args} {
	foreach {ie} "$args" {
		set tp [lindex $ie 0]
		if {$tp == "epref"} {
			set epref [lindex $ie 2]
		}
	}
	global apvar

	if {$apvar == 1} {
		set s "dosig add-party-rej.request $cref 0 default {epref to $epref} {cause 0 31}"
	} else {
		set s "dosig party-alerting.request $cref 0 default {epref to $epref}"
		sleep 200 "ap_ack $cref $epref"
	}
	puts $s
	eval $s
}

proc ap_ack {cref epref} {
	set s "dosig add-party-ack.request $cref 0 default {epref to $epref}"
	puts $s
	eval $s
}

proc dp_ind {cref flag args} {
	puts "DROP-PARTY $cref $flag $args"
	foreach {ie} "$args" {
		set tp [lindex $ie 0]
		if {$tp == "epref"} {
			set epref [lindex $ie 2]
			if {[lindex $ie 1] == "to"} {
				set eflag "from"
			} else {
				set eflag "to"
			}
		}
	}
	puts "EPREF $epref"
	set s "dosig drop-party-ack.request $cref 0 default {epref $eflag $epref} {cause 0 31}"
	puts $s
	eval $s
}
