/*
 * Copyright (c) 2001-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/test_uni_user/exec.c,v 1.1 2003/08/21 11:04:58 hbb Exp $
 *
 * Test user space UNI stack. Execute commands and events.
 */
#include "uniu.h"

/*
 * Execute the current command if it is complete
 */
void
try_exec(Tcl_Interp *interp, Tcl_DString *cmd)
{
	int n;
	const char *e;

	if (!Tcl_CommandComplete(Tcl_DStringValue(cmd)))
		return;

	n = Tcl_RecordAndEval(interp, Tcl_DStringValue(cmd), 0);
	Tcl_DStringFree(cmd);

	if (n != TCL_OK) {
		fprintf(stderr, "error: %s\n", Tcl_GetStringResult(interp));
		if (errinfo &&
		    (e = Tcl_GetVar(interp, "errorInfo",
		    TCL_GLOBAL_ONLY)) != NULL)
			fprintf(stderr, "%s\n", e);

	} else if (*(e = Tcl_GetStringResult(interp)) != '\0')
		fprintf(stderr, "%s\n", e);

	doprompt = 1;
}

/*
 * Evaluate an event
 */
void
try_event(Tcl_Interp *interp, Tcl_DString *str)
{
	int res;
	const char *e;

	if ((res = Tcl_GlobalEval(interp, Tcl_DStringValue(str))) != TCL_OK) {
		fprintf(stderr, "error evaluating event '%s': %s\n",
		    Tcl_DStringValue(str), Tcl_GetStringResult(interp));
		if (errinfo &&
		    (e = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY)))
			fprintf(stderr, "%s\n", e);
	}
	Tcl_DStringFree(str);
}

void
set_error(Tcl_Interp *interp, const char *fmt, ...)
{
	va_list ap;
	char *ret;

	va_start(ap, fmt);
	vasprintf(&ret, fmt, ap);
	if (ret == NULL)
		err(1, "vasprintf");
	va_end(ap);
	Tcl_SetResult(interp, ret, TCL_VOLATILE);
	free(ret);
}
