/*
 * Copyright (c) 2002-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/unidecode/unidecode.c,v 1.3 2004/01/02 15:15:57 hbb Exp $
 *
 * Program to decode UNI messages.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <err.h>

#define err foo_err
#define index foo_index
#include <tcl.h>
#undef index
#undef err

#include <netnatm/unimsg.h>
#include <netnatm/addr.h>
#include <netnatm/msg/unistruct.h>
#include <netnatm/msg/unimsglib.h>
#include "unitcl.h"

static const char usgtxt[] = "\
Usage: unidecode [-dhmp] [msg-file]\n\
  -d	dump message in hex\n\
  -h	print this help text\n\
  -m	increment multiline flag for print routines\n\
  -p	use print routines instead of tcl\
";

int
main(int argc, char *argv[])
{
	int res, opt, doprint = 0, dump = 0;
	size_t ret;
	u_int n;
	struct uni_msg *msg;
	struct uni_all uni;
	Tcl_DString str;
	Tcl_Interp *interp;
	struct unicx cx;
	char buf[20000];

	uni_initcx(&cx);
	cx.q2932 = 2;

	while ((opt = getopt(argc, argv, "dhmp")) != -1)
		switch (opt) {

		  case 'd':
			dump++;
			break;

		  case 'h':
			fprintf(stderr, "%s\n", usgtxt);
			exit(0);

		  case 'm':
			cx.multiline++;
			break;

		  case 'p':
			doprint++;
			break;
		}

	argc -= optind;
	argv += optind;

	if (argc > 1)
		errx(1, "too many arguments\n%s", usgtxt);

	if (argc == 1 && strcmp(argv[0], "-") != 0 &&
	    freopen(argv[0], "r", stdin) == NULL)
		err(1, "%s", argv[0]);

	if ((msg = uni_msg_alloc(1000)) == NULL)
		err(1, NULL);
	if ((ssize_t)(ret = read(STDIN_FILENO, msg->b_rptr,
	    uni_msg_space(msg))) == -1)
		err(1, "read");
	msg->b_wptr = msg->b_rptr + ret;

	if (dump) {
		printf("%zu: ", uni_msg_len(msg));
		for (n = 0; n < uni_msg_len(msg); n++)
			printf("%02x ", msg->b_rptr[n]);
		printf("\n");
	}

	if ((res = uni_decode(msg, &uni, &cx)) != 0)
		errx(1, "decoding error: %d", res);

	if (doprint) {
		uni_print(buf, sizeof(buf), &uni, &cx);
		printf("%s\n", buf);
	} else {
		interp = Tcl_CreateInterp();
		Tcl_DStringInit(&str);
		if (fmt_mtype(interp, &str, uni.mtype) != TCL_OK)
			errx(1, "fmt_mtype failed: %s",
			    Tcl_GetStringResult(interp));
		fmt_cref(&str, &uni.u.hdr.cref);
		if (fmt_msg(interp, &str, &uni) != TCL_OK)
			errx(1, "fmt_msg failed: %s",
			    Tcl_GetStringResult(interp));
		printf("%s\n", Tcl_DStringValue(&str));
	}

	return (0);
}
