/*
 * Copyright (c) 1998-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/uniencode/uniencode.c,v 1.4 2004/01/02 15:15:57 hbb Exp $
 *
 * Program to encode UNI messages.
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <err.h>

#define index foo_index
#define err foo_err
#include <tcl.h>
#undef index
#undef err

#include <netnatm/unimsg.h>
#include <netnatm/addr.h>
#include <netnatm/msg/unistruct.h>
#include <netnatm/msg/unimsglib.h>
#include "unitcl.h"

static const char usgtxt[] = "\
Usage: uniencode [-hmptx] msg >out\n\
Flags:\n\
  -h	print this help text\n\
  -m	increment multiline printing level\n\
  -p	same as -t but use print routine\n\
  -t	don't emit; decode instead\n\
  -x	print hex bytes";

int
main(int argc, char *argv[])
{
	int opt, res;
	struct uni_msg *msg;
	struct uni_all uni;
	Tcl_Interp *interp;
	Tcl_DString str;
	const char **av;
	int ac;
	struct uni_cref cref;
	struct unicx cx;
	int act = 0;
	u_int i;
	char buf[20000];

	uni_initcx(&cx);
	cx.q2932 = 2;

	while ((opt = getopt(argc, argv, "hmptx")) != -1)
		switch (opt) {

		  case 'h':
			fprintf(stderr, "%s\n", usgtxt);
			return (0);

		  case 'm':
			cx.multiline++;
			break;

		  case 'p':
			act = 2;
			break;

		  case 't':
			act = 1;
			break;

		  case 'x':
			act = 3;
			break;
		}

	argc -= optind;
	argv += optind;

	if (argc != 1)
		errx(1, "bad number of arguments\n%s", usgtxt);

	if ((interp = Tcl_CreateInterp()) == NULL)
		errx(1, NULL);

	if (Tcl_SplitList(interp, argv[0], &ac, &av) != TCL_OK)
		errx(1, "%s", Tcl_GetStringResult(interp));

	if (ac < 1)
		errx(1, "need at least the message type");

	memset(&uni, 0, sizeof(uni));

	if (parse_mtype(interp, av[0], &uni.mtype))
		errx(1, "%s", Tcl_GetStringResult(interp));
	ac--;
	av++;
	if (parse_cref(interp, ac, av, &cref))
		errx(1, "%s", Tcl_GetStringResult(interp));
	ac -= 2;
	av += 2;
	if (ac > 0 && !parse_msgact(interp, av[0], &uni.u.hdr)) {
		ac--;
		av++;
	} else
		uni.u.hdr.act = UNI_MSGACT_DEFAULT;
	if(parse_msg(interp, ac, av, &uni, &cx))
		errx(1, "%s", Tcl_GetStringResult(interp));

	uni.u.hdr.cref = cref;

	if (act == 0) {
		msg = uni_msg_alloc(0);
		res = uni_encode(msg, &uni, &cx);
		write(1, msg->b_rptr, uni_msg_len(msg));
		if (res != 0)
			errx(1, "coding failed: %d", res);

	} else if (act == 1) {
		Tcl_DStringInit(&str);
		if (fmt_mtype(interp, &str, uni.mtype) != TCL_OK)
			warnx("fmt_mtype failed: %s",
			    Tcl_GetStringResult(interp));
		fmt_cref(&str, &uni.u.hdr.cref);
		if (fmt_msg(interp, &str, &uni) != TCL_OK)
			warnx("fmt_msg failed: %s",
			    Tcl_GetStringResult(interp));
		printf("%s\n", Tcl_DStringValue(&str));

	} else if (act == 2) {
		uni_print(buf, sizeof(buf), &uni, &cx);
		printf("%s\n", buf);

	} else if (act == 3) {
		msg = uni_msg_alloc(0);
		if ((res = uni_encode(msg, &uni, &cx)) != 0)
			errx(1, "coding failed: %d", res);

		for(i = 0; i < uni_msg_len(msg); i++) {
			if (i % 16 == 0)
				printf("%04x  ", i);
			printf(" %02x", msg->b_rptr[i]);
			if (i % 16 == 15)
				printf("\n");
		}
		if (i % 16 != 0)
			printf("\n");
	} else
		abort();

	return (0);
}
